//*********************************************************************************
//**
//** Project.........: A menu driven Multi Display RF Power and SWR Meter
//**                   using a Tandem Match Coupler and 2x AD8307.
//**                                 or alternately
//**                   using 2 transformers in an V and I arrangement,
//**                   to feed 2x AD8307 and a MCK12140 based 360 degree
//**                   capable Phase Detector circuit, implementing an
//**                   RF Power/SWR and Phase + Impedance meter
//**
//** Copyright (C) 2014  Loftur E. Jonasson  (tf3lj [at] arrl [dot] net)
//**
//** This program is free software: you can redistribute it and/or modify
//** it under the terms of the GNU General Public License as published by
//** the Free Software Foundation, either version 3 of the License, or
//** (at your option) any later version.
//**
//** This program is distributed in the hope that it will be useful,
//** but WITHOUT ANY WARRANTY; without even the implied warranty of
//** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//** GNU General Public License for more details.
//**
//** You should have received a copy of the GNU General Public License
//** along with this program.  If not, see <http://www.gnu.org/licenses/>.
//**
//** Platform........: AT90usb1286 @ 16MHz
//**
//** Initial version.: 0.50, 2013-09-29  Loftur Jonasson, TF3LJ / VE2LJX
//**                   (beta version)
//**
//** History.........: Check the PM.c file
//**
//**
//*********************************************************************************

#include "PM.h"
#include "lcd.h"
#include <stdio.h>
#include <avr/pgmspace.h>

#define menu_level0_text				0
#define menu_swr_alarm_pwr_thresh_text	1
#define menu_pep_text					2
#define menu_screenthres_text			3
#define menu_serial_text				4
#define menu_calibrate_text				5
#define menu_factory_text				6
#define led_menu_items_text				7
#define led_menu_text					8

#define no_change						0
#define store_change					1

const char __flash SWR_menu_item0[] PROGMEM = "SWR Alarm Threshold:";
const char __flash SWR_menu_item1[] PROGMEM = "Adjust->   ";
const char __flash SWR_menu_item2[] PROGMEM = "Range is 1.5 to 3.9";
const char __flash SWR_menu_item3[] PROGMEM = "4.0 = SWR Alarm Off";

const __flash char* const __flash SWR_menu_text[] =
{	SWR_menu_item0, SWR_menu_item1, SWR_menu_item2, SWR_menu_item3 };


// Flags for SWR Alarm & Threshold menu
#define SWR_ALARM_THRESHOLD		1
#define	SWR_ALARM_PWR_THRESHOLD	2

// First Level Menu Items
const uint8_t level0_menu_size = 11;
const char __flash swr_string_1[] PROGMEM ="1 SWR Alarm";
const char __flash swr_string_2[] PROGMEM ="2 SWR Alarm Power";
const char __flash swr_string_3[] PROGMEM ="3 PEP Period";
const char __flash swr_string_4[] PROGMEM ="4 Screensaver";
const char __flash swr_string_5[] PROGMEM ="5 Scale Ranges";
const char __flash swr_string_6[] PROGMEM ="6 Serial Data";
const char __flash swr_string_7[] PROGMEM ="7 Calibrate";
const char __flash swr_string_8[] PROGMEM ="8 Debug Display";
const char __flash swr_string_9[] PROGMEM ="9 Reset to Default";
const char __flash swr_string_10[] PROGMEM ="10 LCD Light";
const char __flash swr_string_11[] PROGMEM ="0 Exit";

// SWR Alarm Power Threshold menu Items
const uint8_t swr_alarm_pwr_thresh_menu_size = 5;
const char __flash swr_alarm_string_1[] PROGMEM ="     1mW";
const char __flash swr_alarm_string_2[] PROGMEM ="    10mW";
const char __flash swr_alarm_string_3[] PROGMEM ="    0,1W";
const char __flash swr_alarm_string_4[] PROGMEM ="      1W";
const char __flash swr_alarm_string_5[] PROGMEM ="     10W";

const char	__flash	swr_alarm_string_a[] PROGMEM="SWR Alarm Power:";
const char	__flash	swr_alarm_string_b[] PROGMEM="Select";
const char	__flash	swr_alarm_string_c[] PROGMEM="Available thresholds";
const char	__flash	swr_alarm_string_d[] PROGMEM="1mW 10mW 0.1W 1W 10W";

// PEP Sample Period select menu Items
const uint8_t pep_menu_size = 3;
const char __flash pep_menu_string1[] PROGMEM ="      1s";
const char __flash pep_menu_string2[] PROGMEM ="    2.5s";
const char __flash pep_menu_string3[] PROGMEM ="      5s";

const char __flash pep_menu_stringa[] PROGMEM ="PEP sampling period:";
const char __flash pep_menu_stringb[] PROGMEM ="Available periods";
const char __flash pep_menu_stringc[] PROGMEM ="1, 2.5 or 5 seconds";

// Screensaver Threshold select menu Items
const uint8_t screenthresh_menu_size = 6;
const char __flash screenthresh_menu_items_string1[] PROGMEM ="     Off";
const char __flash screenthresh_menu_items_string2[] PROGMEM ="     1uW";
const char __flash screenthresh_menu_items_string3[] PROGMEM ="    10uW";
const char __flash screenthresh_menu_items_string4[] PROGMEM ="   100uW";
const char __flash screenthresh_menu_items_string5[] PROGMEM ="     1mW";
const char __flash screenthresh_menu_items_string6[] PROGMEM ="    10mW";

const char __flash screenthresh_menu_items_stringa[] PROGMEM ="ScreensaverThreshld:";
const char __flash screenthresh_menu_items_stringb[] PROGMEM ="Available thresholds";
const char __flash screenthresh_menu_items_stringc[] PROGMEM ="1uW-10mW,  10x steps";

// Serial menu Items
const uint8_t serial_menu_size = 2;
const char	__flash serial_menu_string1[] PROGMEM ="    Off";
const char	__flash serial_menu_string2[] PROGMEM ="     On";

// Calibrate menu Items
const uint8_t calibrate_menu_size = 7;
const char	__flash calibrate_menu_string1[] PROGMEM="OneLevelCal(dBm)";
const char	__flash calibrate_menu_string2[] PROGMEM=" 1st Level (dBm)";
const char	__flash calibrate_menu_string3[] PROGMEM=" 2nd Level (dBm)";
//const char	__flash calibrate_menu_string4[] PROGMEM="OneLevelCal(dBm)";
//const char	__flash calibrate_menu_string5[] PROGMEM=" 1st Level (dBm)";
//const char	__flash calibrate_menu_string6[] PROGMEM=" 2nd Level (dBm)";
const char	__flash calibrate_menu_string7[] PROGMEM="         Go back";

// Factory Reset menu Items
const uint8_t factory_menu_size = 2;
const char	__flash factory_menu_string1[] PROGMEM=" No  - Go back";
const char	__flash factory_menu_string2[] PROGMEM=" Yes - Reset";

// LED backlight intensity menu items
const uint8_t led_menu_size = 5;
/*const char	__flash led_menu_string1[] PROGMEM="  Off";
const char	__flash led_menu_string2[] PROGMEM="   On";*/
const char	__flash led_menu_string1[] PROGMEM="   0%";
const char	__flash led_menu_string2[] PROGMEM="  25%";
const char	__flash led_menu_string3[] PROGMEM="  50%";
const char	__flash led_menu_string4[] PROGMEM="  75%";
const char	__flash led_menu_string5[] PROGMEM=" 100%";


// General text
const char __flash nothing_changed_string[] PROGMEM = "Nothing Changed";
const char __flash value_stord_strin[]		PROGMEM = "Value Stored";

const __flash char* const __flash general_text[]=
{
	nothing_changed_string, value_stord_strin
};

const __flash char* const __flash menu_items_text[][11] =
{	
	{swr_string_1, swr_string_2, swr_string_3, swr_string_4, swr_string_5,
	swr_string_6, swr_string_7, swr_string_8, swr_string_9, swr_string_10, swr_string_11},

	{swr_alarm_string_1, swr_alarm_string_2, swr_alarm_string_3, swr_alarm_string_4, swr_alarm_string_5},
		
	{pep_menu_string1, pep_menu_string2, pep_menu_string3},
	
	{screenthresh_menu_items_string1, screenthresh_menu_items_string2, screenthresh_menu_items_string3, 
	 screenthresh_menu_items_string4, screenthresh_menu_items_string5, screenthresh_menu_items_string6,},
	 
	{serial_menu_string1, serial_menu_string2},
		
	{calibrate_menu_string1, calibrate_menu_string2, calibrate_menu_string3, calibrate_menu_string1, calibrate_menu_string2,
	 calibrate_menu_string3, calibrate_menu_string7},
	 
	{factory_menu_string1, factory_menu_string2},
		
	{led_menu_string1, led_menu_string2, led_menu_string3, led_menu_string4, led_menu_string5}
};

const __flash char* const __flash in_menu_text[][4] =
{
	{swr_alarm_string_a, swr_alarm_string_b, swr_alarm_string_c, swr_alarm_string_d},
		
	{pep_menu_stringa, pep_menu_stringb, pep_menu_stringc},
		
	{screenthresh_menu_items_stringa, screenthresh_menu_items_stringb, screenthresh_menu_items_stringc}
};

// Flag for PEP Sample Period select
#define PEP_MENU		3

// Flag for Screensaver Threshold select menu
#define SCREENTHRESH_MENU	5

// Flag for Encoder Resolution Change
//#define ENCODER_MENU	8

// Flag for Scale Range menu
#define SCALERANGE_MENU	6

// Flags for Scale Range Submenu functions
#define SCALE_SET0_MENU	600
#define SCALE_SET1_MENU	601
#define SCALE_SET2_MENU	602

// Flag for Serial Data Out
#define SERIAL_MENU		7

// Flag for Calibrate menu
#define CAL_MENU		8

// Flags for Calibrate Submenu functions
#define CAL_SET0_MENU	800	// Single level calibration
#define CAL_SET1_MENU	801	// 1st level
#define CAL_SET2_MENU	802	// 2nd level
#define CAL_SET3_MENU	803	// Single level calibration V/UHF
#define CAL_SET4_MENU	804	// 1st level V/UHF
#define CAL_SET5_MENU	805	// 2nd level V/UHF

// Flag for Debug Screen
#define DEBUG_MENU		9

// Flag for Factory Reset
#define FACTORY_MENU	11

// Flag for LED backlight intensity
#define LED_LIGHT		12


uint16_t		menu_level = 0;						// Keep track of which menu we are in
uint8_t			menu_data = 0;						// Pass data to lower menu

int8_t			gain_selection;						// keep track of which GainPreset is currently selected

//----------------------------------------------------------------------
// Display a Menu of choices, one line at a time
//
// menu refers to a pointer array containing the Menu to be printed
//
// menu_size indicates how many pointers (menu items) there are in the array
//
// current_choice indicates which item is currently up for selection if
// pushbutton is pushed
//
// begin row, begin_col indicate the upper left hand corner of the one, three or 
// four lines to be printed
//
// lines indicates how many lines are displayed at any one time, 1, 3 or 4.
//----------------------------------------------------------------------
void lcd_scroll_Menu(uint8_t menu, uint8_t menu_size, uint8_t current_choice, uint8_t begin_row, uint8_t begin_col, uint8_t lines)
{
	uint8_t a, x;
	char buffer[20];

	// Clear LCD from begin_col to end of line.
	lcdGotoXY(begin_col, begin_row);
	for (a = begin_col; a < 20; a++)
		lcdPrintData(" ",1);
	if (lines > 1)
	{
		lcdGotoXY(begin_col, begin_row+1);
		for (a = begin_col; a < 20; a++)
		lcdPrintData(" ",1);
	}
	if (lines > 2)
	{
		lcdGotoXY(begin_col, begin_row+2);
		for (a = begin_col; a < 20; a++)
		lcdPrintData(" ",1);
	}

	// First line contains previous choice, second line contains
	// current choice preceded with a '->', and third line contains
	// next choice
	if (current_choice == 0) 
		x = menu_size - 1;
	else 
		x = current_choice - 1;
	if (lines > 1)
	{
		lcdGotoXY(begin_col + 2, begin_row);
		//sprintf(lcd_buf,"%s", *(menu + x));
		strcpy_P(buffer, menu_items_text[menu][x]);
		lcdPrintData(buffer, strlen(buffer));
		lcdGotoXY(begin_col, begin_row + 1);
		//sprintf(lcd_buf,"->%s", *(menu + current_choice));
		strcpy_P(buffer, menu_items_text[menu][current_choice]);
		lcdPrintData("->", 2);	lcdPrintData(buffer, strlen(buffer));
		if (current_choice == menu_size - 1) 
			x = 0;
		else 
			x = current_choice + 1;

		lcdGotoXY(begin_col + 2, begin_row + 2);
		//sprintf(lcd_buf,"%s", *(menu + x));
		strcpy_P(buffer, menu_items_text[menu][x]);
		lcdPrintData(buffer, strlen(buffer));
	}
	else
	{
		lcdGotoXY(begin_col, begin_row);
		//sprintf(lcd_buf,"->%s", *(menu + current_choice));
		strcpy_P(buffer, menu_items_text[menu][current_choice]);
		lcdPrintData("->", 2); lcdPrintData(buffer, strlen(buffer));
	}
	// LCD print lines 1 to 3
	// 4 line display.  Preformat and print the fourth line as well
	if (lines == 4)
	{
		if (current_choice == menu_size-1) x = 1;
		else if (current_choice == menu_size - 2 ) x = 0;
		else x = current_choice + 2;
		lcdGotoXY(begin_col, begin_row+3);
		for (a = begin_col; a < 20; a++)
		lcdPrintData(" ",1);
		lcdGotoXY(begin_col + 2, begin_row + 3);
		//sprintf(lcd_buf,"  %s", *(menu + x));
		strcpy_P(buffer, menu_items_text[menu][x]);
		lcdPrintData(buffer, strlen(buffer));
	}
}


//----------------------------------------------------------------------
// Menu functions begin:
//----------------------------------------------------------------------


//--------------------------------------------------------------------
// Debug Screen, exit on push
//--------------------------------------------------------------------
void debug_menu(void)
{
	char buffer[21];
	
	lcd_display_debug();				// Display Config and measured input voltages etc...

	// Exit on Button Push
	if (Status & SHORT_PUSH)
	{
		Status &=  ~SHORT_PUSH;			// Clear pushbutton status

		lcdClear();
		lcdGotoXY(1,1);				
		strcpy_P(buffer, general_text[no_change]);
		lcdPrintData(buffer,strlen(buffer));
		_delay_ms(1000);
		//Menu_Mode &= ~CONFIG;			// We're done, EXIT
		Menu_Mode |= CONFIG;			// We're NOT done, just backing off
		menu_level = 0;					// We are done with this menu level
	}
}


//--------------------------------------------------------------------
// SWR Alarm Threshold Set Menu
//--------------------------------------------------------------------
void swr_alarm_threshold_menu(void)
{
	static uint8_t LCD_upd = FALSE;		// Keep track of LCD update requirements
	char buffer[21];
	
	// Selection modified by encoder.  We remember last selection, even if exit and re-entry
	if (Status & ENC_CHANGE)
	{
		if ((encOutput > 0) && (R.SWR_alarm_trig < 40) )  // SWR 4:1 is MAX value, and equals no alarm
		{
			R.SWR_alarm_trig++;
		}
		if ((encOutput < 0) && (R.SWR_alarm_trig > 15) )	// SWR of 1.5:1 is MIN value for SWR alarm
		{
			R.SWR_alarm_trig--;
		}
		// Reset data from Encoder
		Status &=  ~ENC_CHANGE;
		encOutput = 0;
		// Indicate that an LCD update is needed
		LCD_upd = FALSE;
	}

	// If LCD update is needed
	if (LCD_upd == FALSE)
	{
		LCD_upd = TRUE;					// We have serviced LCD

		lcdClear();
		
		lcdGotoXY(0,0);
		strcpy_P(buffer, SWR_menu_text[0]);
		lcdPrintData(buffer,strlen(buffer));
		lcdGotoXY(0,1);
		strcpy_P(buffer, SWR_menu_text[1]);
		lcdPrintData(buffer,strlen(buffer));

		if (R.SWR_alarm_trig == 40)
			lcdPrintData("Off",3);		
		else
		{
			sprintf(lcd_buf,"%1u.%01u",R.SWR_alarm_trig/10, R.SWR_alarm_trig%10);
			lcdPrintData(lcd_buf,strlen(lcd_buf));
		}					
		lcdGotoXY(0,2);
		strcpy_P(buffer, SWR_menu_text[2]);
		lcdPrintData(buffer,strlen(buffer));
		lcdGotoXY(0,3);
		strcpy_P(buffer, SWR_menu_text[3]);
		lcdPrintData(buffer,strlen(buffer));

	}	
	// Enact selection by saving in EEPROM
	if (Status & SHORT_PUSH)
	{
		Status &=  ~SHORT_PUSH;			// Clear pushbutton status

		lcdClear();
		lcdGotoXY(1,1);
		
		if (eeprom_read_byte(&E.SWR_alarm_trig) != R.SWR_alarm_trig)			// New Value
		{
			eeprom_write_block(&R.SWR_alarm_trig, &E.SWR_alarm_trig, sizeof (R.SWR_alarm_trig));
			strcpy_P(buffer, general_text[store_change]);
			lcdPrintData(buffer,strlen(buffer));
		}
		else
		{
			strcpy_P(buffer, general_text[no_change]);
			lcdPrintData(buffer,strlen(buffer));
		}

		_delay_ms(1000);
		Menu_Mode |= CONFIG;			// We're NOT done, just backing off
//		Menu_Mode &= ~CONFIG;			// We're done, EXIT
		menu_level = 0;					// We are done with this menu level
		LCD_upd = FALSE;				// Make ready for next time
	}
}


//--------------------------------------------------------------------
// SWR Alarm Power Threshold Menu
//--------------------------------------------------------------------
void swr_alarm_power_threshold_menu(void)
{
	static int8_t	current_selection;
	static uint8_t	LCD_upd = FALSE;		// Keep track of LCD update requirements
	char buffer[20];
	
	// Get Current value
	// 1mW=>0, 10mW=>1, 100mW=>2, 1000mW=>3, 10000mW=>4
	// current_selection = log10(R.SWR_alarm_pwr_thresh); // This should have worked :(
	if      (R.SWR_alarm_pwr_thresh == 1) current_selection = 0;
	else if (R.SWR_alarm_pwr_thresh == 10) current_selection = 1;
	else if (R.SWR_alarm_pwr_thresh == 100) current_selection = 2;
	else if (R.SWR_alarm_pwr_thresh == 1000) current_selection = 3;
	else if (R.SWR_alarm_pwr_thresh == 10000) current_selection = 4;
	
	// Selection modified by encoder.  We remember last selection, even if exit and re-entry
	if (Status & ENC_CHANGE)
	{
		if (encOutput > 0)
		{
			current_selection++;
		}
		else if (encOutput < 0)
		{
			current_selection--;
		}		

		// Reset data from Encoder
		Status &=  ~ENC_CHANGE;
		encOutput = 0;
		// Indicate that an LCD update is needed
		LCD_upd = FALSE;
	}

	// If LCD update is needed
	if (LCD_upd == FALSE)
	{
		LCD_upd = TRUE;					// We have serviced LCD

//itoa(current_selection, buffer, 10 );
//lcdPrintData(buffer,strlen(buffer));

		// Keep Encoder Selection Within Bounds of the Menu Size
		uint8_t menu_size = swr_alarm_pwr_thresh_menu_size;
		while(current_selection >= menu_size)
			current_selection -= menu_size;
		while(current_selection < 0)
			current_selection += menu_size;

//itoa(current_selection, buffer, 10 );
//lcdPrintData(buffer,strlen(buffer));
//_delay_ms(1000);


		// Update with currently selected value
		R.SWR_alarm_pwr_thresh = pow(10,current_selection);
	
		lcdClear();
		lcdGotoXY(0,0);
		strcpy_P(buffer, in_menu_text[0][0]);
		lcdPrintData(buffer, strlen(buffer));
		lcdGotoXY(0,1);
		strcpy_P(buffer, in_menu_text[0][1]);
		lcdPrintData(buffer, strlen(buffer));


		// Print the Rotary Encoder scroll Menu
		//lcd_scroll_Menu((char**)swr_alarm_pwr_thresh_menu_items, menu_size, current_selection, 1, 6, 1);
		lcd_scroll_Menu(menu_swr_alarm_pwr_thresh_text, swr_alarm_pwr_thresh_menu_size, current_selection, 1, 6, 1);

		lcdGotoXY(0,2);
		strcpy_P(buffer, in_menu_text[0][2]);
		lcdPrintData(buffer, strlen(buffer));
	
		lcdGotoXY(0,3);
		strcpy_P(buffer, in_menu_text[0][3]);
		lcdPrintData(buffer, strlen(buffer));

	}
	
	// Enact selection
	if (Status & SHORT_PUSH)
	{
		lcdClear();
		lcdGotoXY(1,1);
		Status &=  ~SHORT_PUSH;								// Clear pushbutton status
		// Check if selected threshold is not same as previous
		if (eeprom_read_word(&E.SWR_alarm_pwr_thresh) != R.SWR_alarm_pwr_thresh)// New Value
		{
			eeprom_write_block(&R.SWR_alarm_pwr_thresh, &E.SWR_alarm_pwr_thresh, sizeof (R.SWR_alarm_pwr_thresh));
			strcpy_P(buffer, general_text[store_change]);
			lcdPrintData(buffer,strlen(buffer));
		}
		else 
		{
			strcpy_P(buffer, general_text[no_change]);
			lcdPrintData(buffer,strlen(buffer));
		}
//		Menu_Mode &= ~CONFIG;								// We're done, EXIT
		Menu_Mode |= CONFIG;								// We're NOT done, just backing off
		menu_level = 0;										// We are done with this menu level
		_delay_ms(1000);
		LCD_upd = FALSE;									// Make ready for next time
	}
}


//--------------------------------------------------------------------
// Peak Envelope Power (PEP) period selection Menu
//--------------------------------------------------------------------
void pep_menu(void)
{
	static int8_t	current_selection;
	static uint8_t	LCD_upd = FALSE;	// Keep track of LCD update requirements
	char buffer[20];
		
	// Get Current value
	if (R.PEP_period == 500) current_selection = 1;			// 2.5 seconds
	else if (R.PEP_period == 1000) current_selection = 2;	// 5 seconds
	else current_selection = 0;								// Any other value, other than 1s, is invalid
	
	// Selection modified by encoder.  We remember last selection, even if exit and re-entry
	if (Status & ENC_CHANGE)
	{
		if (encOutput > 0)
		{
			current_selection++;
		}
		else if (encOutput < 0)
		{
			current_selection--;
		}
		// Reset data from Encoder
		Status &=  ~ENC_CHANGE;
		encOutput = 0;

		// Indicate that an LCD update is needed
		LCD_upd = FALSE;
	}

	// If LCD update is needed
	if (LCD_upd == FALSE)
	{
		LCD_upd = TRUE;					// We have serviced LCD

		// Keep Encoder Selection Within Bounds of the Menu Size
		uint8_t menu_size = pep_menu_size;
		while(current_selection >= menu_size)
			current_selection -= menu_size;
		while(current_selection < 0)
			current_selection += menu_size;

		if      (current_selection == 1) R.PEP_period = 500;
		else if (current_selection == 2) R.PEP_period = 1000;
		else R.PEP_period = 200;			
		
		lcdClear();
		lcdGotoXY(0,0);
		strcpy_P(buffer, in_menu_text[1][0]);
		lcdPrintData(buffer, strlen(buffer));

		lcdGotoXY(0,1);
		strcpy_P(buffer, in_menu_text[0][1]);	// select
		lcdPrintData(buffer, strlen(buffer));

		// Print the Rotary Encoder scroll Menu
		//lcd_scroll_Menu((char**)pep_menu_items, menu_size, current_selection, 1, 6, 1);
		lcd_scroll_Menu(menu_pep_text, pep_menu_size, current_selection,1 ,6 ,1);
	
		lcdGotoXY(0,2);
		strcpy_P(buffer, in_menu_text[1][1]);
		lcdPrintData(buffer, strlen(buffer));

		lcdGotoXY(0,3);
		strcpy_P(buffer, in_menu_text[1][2]);
		lcdPrintData(buffer, strlen(buffer));
	}

	// Enact selection
	if (Status & SHORT_PUSH)
	{
		lcdClear();
		lcdGotoXY(1,1);
		
		Status &=  ~SHORT_PUSH;			// Clear pushbutton status
		
		// Check if selected threshold is not same as previous
		if (eeprom_read_word(&E.PEP_period) != R.PEP_period)
		{
			eeprom_write_block(&R.PEP_period, &E.PEP_period, sizeof (R.PEP_period));
			strcpy_P(buffer, general_text[store_change]);
			lcdPrintData(buffer,strlen(buffer));
		}
		else 
		{
			strcpy_P(buffer, general_text[no_change]);
			lcdPrintData(buffer,strlen(buffer));
		}

		_delay_ms(1000);
//		Menu_Mode &= ~CONFIG;			// We're done with Menu, EXIT
		Menu_Mode |= CONFIG;			// We're NOT done, just backing off
		menu_level = 0;					// We are done with this menu level
		LCD_upd = FALSE;				// Make ready for next time
	}
}


//--------------------------------------------------------------------
// Screensaver Threshold Sensitivity selection Menu
//--------------------------------------------------------------------
void screenthresh_menu(void)
{
	static int8_t	current_selection;
	static uint8_t	LCD_upd = FALSE;		// Keep track of LCD update requirements
	char buffer[20];
		
	// Get Current value
	// 0=0, 1uW=0.001=>1, 10uW=0.01=>2... 100uW=>3, 1mW=>4, 10mW=>5
	if (R.idle_disp_thresh == 0) 
		current_selection = 0;
	else 
		current_selection = log10(R.idle_disp_thresh) + 4;
	
	// Selection modified by encoder.  We remember last selection, even if exit and re-entry
	if (Status & ENC_CHANGE)
	{
		if (encOutput > 0)
		{
			current_selection++;
		}
		else if (encOutput < 0)
		{
			current_selection--;
		}
		// Reset data from Encoder
		Status &=  ~ENC_CHANGE;
		encOutput = 0;

		// Indicate that an LCD update is needed
		LCD_upd = FALSE;
	}

	// If LCD update is needed
	if (LCD_upd == FALSE)
	{
		LCD_upd = TRUE;					// We have serviced LCD

		// Keep Encoder Selection Within Bounds of the Menu Size
		uint8_t menu_size = screenthresh_menu_size;
		while(current_selection >= menu_size)
		current_selection -= menu_size;
		while(current_selection < 0)
		current_selection += menu_size;

		// Update with currently selected value
		// 0=0, 1uW=0.001=>1, 10uW=0.01=>2... 100uW=>3, 1mW=>4, 10mW=>5
		if (current_selection == 0) R.idle_disp_thresh = 0;
		else R.idle_disp_thresh = pow(10,current_selection - 4);
		
		lcdClear();
		lcdGotoXY(0,0);
//		lcdPrintData("ScreensaverThreshld:",20);
		strcpy_P(buffer, in_menu_text[2][0]);
		lcdPrintData(buffer, strlen(buffer));
		lcdGotoXY(0,1);
//		lcdPrintData("Select",6);
		strcpy_P(buffer, in_menu_text[0][1]);
		lcdPrintData(buffer, strlen(buffer));


		// Print the Rotary Encoder scroll Menu
		//lcd_scroll_Menu((char**)screenthresh_menu_items, menu_size, current_selection, 1, 6,1);
		lcd_scroll_Menu(menu_screenthres_text, screenthresh_menu_size, current_selection, 1, 6, 1);

		lcdGotoXY(0,2);
		strcpy_P(buffer, in_menu_text[2][1]);
		lcdPrintData(buffer, strlen(buffer));

		lcdGotoXY(0,3);
		strcpy_P(buffer, in_menu_text[2][2]);
		lcdPrintData(buffer, strlen(buffer));
	}
	
	// Enact selection
	if (Status & SHORT_PUSH)
	{
		lcdClear();
		lcdGotoXY(1,1);
		
		Status &=  ~SHORT_PUSH;			// Clear pushbutton status

		// Check if selected threshold is not same as previous
		if (eeprom_read_float(&E.idle_disp_thresh) != R.idle_disp_thresh)// New Value
		{
			eeprom_write_block(&R.idle_disp_thresh, &E.idle_disp_thresh, sizeof (R.idle_disp_thresh));
			strcpy_P(buffer, general_text[store_change]);
			lcdPrintData(buffer,strlen(buffer));

		}
		else 
		{
			strcpy_P(buffer, general_text[no_change]);
			lcdPrintData(buffer,strlen(buffer));
		}
		_delay_ms(1000);
//		Menu_Mode &= ~CONFIG;			// We're done, EXIT
		Menu_Mode |= CONFIG;			// We're NOT done, just backing off
		menu_level = 0;					// We are done with this menu level
		LCD_upd = FALSE;				// Make ready for next time
	}
}


//--------------------------------------------------------------------
// Scale Range Setup Submenu functions
//--------------------------------------------------------------------
void scalerange_menu_level2(void)
{
	static int16_t	current_selection;	// Keep track of current LCD menu selection
	static uint8_t LCD_upd = FALSE;		// Keep track of LCD update requirements
	char buffer[20];

	uint8_t scale_set;					// Determine whether CAL_SET0, CAL_SET1 or CAL_SET2

	if (menu_level == SCALE_SET0_MENU) scale_set = 0;		// SCALE_SET0_MENU
	else if (menu_level == SCALE_SET1_MENU) scale_set = 1;	// SCALE_SET1_MENU
	else scale_set = 2;										// SCALE_SET2_MENU
	
	
	// Get Current value
	current_selection = R.ScaleRange[scale_set];

	// Selection modified by encoder.  We remember last selection, even if exit and re-entry
	if (Status & ENC_CHANGE)
	{
		if (encOutput > 0)
		{
			current_selection++;
			// Reset data from Encoder
			Status &=  ~ENC_CHANGE;
			encOutput = 0;
		}
		else if (encOutput < 0)
		{
			current_selection--;
			// Reset data from Encoder
			Status &=  ~ENC_CHANGE;
			encOutput = 0;
		}
		// Indicate that an LCD update is needed
		LCD_upd = FALSE;
	}


	// If LCD update is needed
	if (LCD_upd == FALSE)
	{
		LCD_upd = TRUE;								// We are about to have serviced LCD

		// Keep Encoder Selection Within Scale Range Bounds
		int16_t max_range = R.ScaleRange[0] * 10;	// Highest permissible Scale Range for ranges 2 and 3,
		if (max_range > 99) max_range = 99;			// never larger than 10x range 1 and never larger than 99
		int16_t min_range = R.ScaleRange[0];		// Lowest permissible Scale Range for ranges 2 and 3,
													// never smaller than range 1
		if (scale_set==0)
		{
			// Set bounds for Range 1 adjustments
			if(current_selection > 99) current_selection = 99;	// Range 1 can take any value between 1 and 99
			if(current_selection < 1) current_selection =	1;			
		}
		if (scale_set>0)
		{
			// Set bounds for Range 2 and 3 adjustments
			if(current_selection > max_range) current_selection = max_range;
			if(current_selection < min_range) current_selection = min_range;
		}
		// Store Current value in running storage
		R.ScaleRange[scale_set] = current_selection;

		//
		// Bounds dependencies check and adjust
		//
		// Ranges 2 and 3 cannot ever be larger than 9.9 times Range 1
		// Range 2 is equal to or larger than Range 1
		// Range 3 is equal to or larger than Range 2
		// If two Ranges are equal, then only two Scale Ranges in effect
		// If all three Ranges are equal, then only one Range is in effect
		// If Range 1 is being adjusted, Ranges 2 and 3 can be pushed up or down as a consequence
		// If Range 2 is being adjusted up, Range 3 can be pushed up
		// If Range 3 is being adjusted down, Range 2 can be pushed down
		if (R.ScaleRange[1] >= R.ScaleRange[0]*10) R.ScaleRange[1] = R.ScaleRange[0]*10 - 1;
		if (R.ScaleRange[2] >= R.ScaleRange[0]*10) R.ScaleRange[2] = R.ScaleRange[0]*10 - 1;
		// Ranges 2 and 3 cannot be smaller than Range 1			
		if (R.ScaleRange[1] < R.ScaleRange[0]) R.ScaleRange[1] = R.ScaleRange[0];
		if (R.ScaleRange[2] < R.ScaleRange[0]) R.ScaleRange[2] = R.ScaleRange[0];

		// Adjustment up of Range 2 can push Range 3 up
		if ((scale_set == 1) && (R.ScaleRange[1] > R.ScaleRange[2])) R.ScaleRange[2] = R.ScaleRange[1];
		// Adjustment down of Range 3 can push Range 2 down:
		if ((scale_set == 2) && (R.ScaleRange[2] < R.ScaleRange[1])) R.ScaleRange[1] = R.ScaleRange[2];

		lcdClear();
		
		// Populate the Display - including current values selected for scale ranges
		lcdGotoXY(0,0);
		lcdPrintData("Adjust, Push to Set:",20);
		
		lcdGotoXY(6,1);
		sprintf(lcd_buf,"1st Range = %2u",R.ScaleRange[0]);
		lcdPrintData(lcd_buf,strlen(lcd_buf));
		lcdGotoXY(6,2);
		sprintf(lcd_buf,"2nd Range = %2u",R.ScaleRange[1]);
		lcdPrintData(lcd_buf,strlen(lcd_buf));
		lcdGotoXY(6,3);
		sprintf(lcd_buf,"3rd Range = %2u",R.ScaleRange[2]);
		lcdPrintData(lcd_buf,strlen(lcd_buf));

		// Place "===>" in front of the "ScaleRange" currently being adjusted
		lcdGotoXY(0,scale_set+1);
		lcdPrintData("===>",4);
	}
	
	// Enact selection by saving in EEPROM
	if (Status & SHORT_PUSH)
	{
		Status &=  ~SHORT_PUSH;				// Clear pushbutton status
		lcdClear();
		lcdGotoXY(1,1);

		// Save modified value
		// There are so many adjustable values that it is simplest just to assume
		// a value has always been modified.  Save all 3
		eeprom_write_block(&R.ScaleRange, &E.ScaleRange, sizeof (R.ScaleRange));
		strcpy_P(buffer, general_text[store_change]);
		lcdPrintData(buffer,strlen(buffer));
		_delay_ms(1000);
		Menu_Mode |= CONFIG;				// We're NOT done, just backing off
		menu_level = SCALERANGE_MENU;		// We are done with this menu level
		LCD_upd = FALSE;					// Make ready for next time
	}
}



//--------------------------------------------------------------------
// Scale Range Menu functions
//--------------------------------------------------------------------
void scalerange_menu(void)
{
	static int8_t	current_selection;		// Keep track of current LCD menu selection
	static uint8_t LCD_upd = FALSE;			// Keep track of LCD update requirements

	// Selection modified by encoder.  We remember last selection, even if exit and re-entry
	if (Status & ENC_CHANGE)
	{
		if (encOutput > 0)
		{
			current_selection++;
			// Reset data from Encoder
			Status &=  ~ENC_CHANGE;
			encOutput = 0;
		}
		else if (encOutput < 0)
		{
			current_selection--;
			// Reset data from Encoder
			Status &=  ~ENC_CHANGE;
			encOutput = 0;
		}
		// Indicate that an LCD update is needed
		LCD_upd = FALSE;
	}

	// If LCD update is needed
	if (LCD_upd == FALSE)
	{
		LCD_upd = TRUE;						// We have serviced LCD

		// Keep Encoder Selection Within Bounds of the Menu Size
		uint8_t menu_size = 4;
		while(current_selection >= menu_size)
		current_selection -= menu_size;
		while(current_selection < 0)
		current_selection += menu_size;

		lcdClear();
		
		// Populate the Display - including current values selected for scale ranges
		lcdGotoXY(0,0);
		if (current_selection<3)
			lcdPrintData("Select Scale Range:",19);
		else
			lcdPrintData("Turn or Push to Exit",20);
		
		lcdGotoXY(6,1);
		sprintf(lcd_buf,"1st Range = %2u",R.ScaleRange[0]);
		lcdPrintData(lcd_buf,strlen(lcd_buf));
		lcdGotoXY(6,2);
		sprintf(lcd_buf,"2nd Range = %2u",R.ScaleRange[1]);
		lcdPrintData(lcd_buf,strlen(lcd_buf));
		lcdGotoXY(6,3);
		sprintf(lcd_buf,"3rd Range = %2u",R.ScaleRange[2]);
		lcdPrintData(lcd_buf,strlen(lcd_buf));

		// Place "->" in front of the relevant "ScaleRange" to be selected with a push
		if (current_selection<3)
		{
			lcdGotoXY(4,current_selection+1);
			lcdPrintData("->",2);
		}
	}

	// Enact selection
	if (Status & SHORT_PUSH)
	{
		Status &=  ~SHORT_PUSH;				// Clear pushbutton status

		switch (current_selection)
		{
			case 0:
				menu_level = SCALE_SET0_MENU;
				LCD_upd = FALSE;			// force LCD reprint
				break;
			case 1:
				menu_level = SCALE_SET1_MENU;
				LCD_upd = FALSE;			// force LCD reprint
				break;
			case 2:
				menu_level = SCALE_SET2_MENU;
				LCD_upd = FALSE;			// force LCD reprint
				break;
			default:
				lcdClear();
				lcdGotoXY(0,1);
				lcdPrintData("Done w. Scale Ranges",20);
				_delay_ms(1000);
//				Menu_Mode &=  ~CONFIG;		// We're done
				Menu_Mode |= CONFIG;		// We're NOT done, just backing off
				menu_level = 0;				// We are done with this menu level
				LCD_upd = FALSE;			// Make ready for next time
		}
	}
}


//--------------------------------------------------------------------
// USB Serial Data output ON/OFF
//--------------------------------------------------------------------
void serial_menu(void)
{
	static int8_t	current_selection;
	static uint8_t	LCD_upd = FALSE;		// Keep track of LCD update requirements
	char buffer[20];
	
	current_selection = R.USB_data;
	
	// Selection modified by encoder.  We remember last selection, even if exit and re-entry
	if (Status & ENC_CHANGE)
	{
		if (encOutput > 0)
		{
			current_selection++;
		}
		else if (encOutput < 0)
		{
			current_selection--;
		}
		// Reset data from Encoder
		Status &=  ~ENC_CHANGE;
		encOutput = 0;

		// Indicate that an LCD update is needed
		LCD_upd = FALSE;
	}

	// If LCD update is needed
	if (LCD_upd == FALSE)
	{
		LCD_upd = TRUE;					// We have serviced LCD

		// Keep Encoder Selection Within Bounds of the Menu Size
		uint8_t menu_size = serial_menu_size;
		while(current_selection >= menu_size)
			current_selection -= menu_size;
		while(current_selection < 0)
			current_selection += menu_size;

		R.USB_data = current_selection;
		
		lcdClear();
		lcdGotoXY(0,0);
		lcdPrintData("USB Virtual Com Port",20);
		lcdGotoXY(0,1);
		lcdPrintData("Select",6);

		// Print the Rotary Encoder scroll Menu
		//lcd_scroll_Menu((char**)serial_menu_items, menu_size, current_selection, 1, 0,1);
		lcd_scroll_Menu(menu_serial_text, serial_menu_size, current_selection,1, 0, 1);
	
		lcdGotoXY(0,2);
		lcdPrintData("SerialCommunications",20);
		lcdGotoXY(0,3);
		lcdPrintData("over USB",8);
	}

	// Enact selection
	if (Status & SHORT_PUSH)
	{
		Status &=  ~SHORT_PUSH;						// Clear pushbutton status
		// Check if selected threshold is not same as previous
		if (eeprom_read_byte(&E.USB_data) != R.USB_data)
		{
			eeprom_write_block(&R.USB_data, &E.USB_data, sizeof (R.USB_data));
			lcdClear();
			lcdGotoXY(1,1);
			strcpy_P(buffer, general_text[store_change]);
			lcdPrintData(buffer,strlen(buffer));
			_delay_ms(1000);
			lcdGotoXY(2,3);
			if (R.USB_data==0)
				lcdPrintData("Serial Data is OFF",18);
			else
				lcdPrintData("Serial Data is ON",17);
			lcdGotoXY(2,2);
			lcdPrintData("Reset Meter Now!!!",18);
			_delay_ms(1000);
			asm volatile("jmp 0x00000");	// Soft Reset
			//while (1);					// Bye bye, Death by Watchdog
			//								// BUG, Watchdog is unreliable
			//								// problem related to teensy bootloader?
		}
		else
		{
			lcdClear();
			lcdGotoXY(1,1);
			strcpy_P(buffer, general_text[no_change]);
			lcdPrintData(buffer,strlen(buffer));
			lcdGotoXY(1,2);
			if (R.USB_data==0)
				lcdPrintData("Serial Data is OFF",18);
			else
				lcdPrintData("Serial Data is ON",17);
			_delay_ms(1000);
//			Menu_Mode &=  ~CONFIG;		// We're done
			Menu_Mode |= CONFIG;		// We're NOT done, just backing off
			menu_level = 0;				// We are done with this menu level
			LCD_upd = FALSE;			// Make ready for next time
		}
	}
}


/*
//--------------------------------------------------------------------
// Rotary Encoder Resolution
//--------------------------------------------------------------------
void encoder_menu(void)
{

	uint8_t	current_selection;			// Keep track of current Encoder Resolution

	static uint8_t LCD_upd = FALSE;		// Keep track of LCD update requirements

	// Get Current value
	current_selection = R.encoderRes;

	// Selection modified by encoder.  We remember last selection, even if exit and re-entry
	if (Status & ENC_CHANGE)
	{
		if (encOutput > 0)
		{
			current_selection=current_selection<<1;
	    	// Reset data from Encoder
			Status &=  ~ENC_CHANGE;
			encOutput = 0;
		}
		else if (encOutput < 0)
		{
			current_selection=current_selection>>1;
	  	  	// Reset data from Encoder
			Status &=  ~ENC_CHANGE;
			encOutput = 0;
		}
		// Indicate that an LCD update is needed
		LCD_upd = FALSE;
	}

	// If LCD update is needed
	if (LCD_upd == FALSE)
	{
		LCD_upd = TRUE;					// We have serviced LCD

		// Keep Encoder Selection Within Bounds of the Menu Size
		if(current_selection > 128) current_selection = 128;
		if(current_selection < 1) current_selection = 1;

		// Store Current value in running storage
		R.encoderRes = current_selection;

		lcd_clrscr();
		lcd_gotoxy(0,0);	
		lcd_puts_P("Encoder ResDivide:");

		lcd_gotoxy(0,1);
		lcd_puts_P("Rotate to Adjust");
		lcd_gotoxy(0,2);
		lcd_puts_P("Push to Save");
		// Format and print current value
		lcd_gotoxy(0,3);
		lcd_puts_P("->");

		int16_t val = current_selection;
		rprintf("%3u",val);
	}

	// Enact selection by saving in EEPROM
	if (Status & SHORT_PUSH)
	{
		Status &=  ~SHORT_PUSH;			// Clear pushbutton status

		// Save modified value
		eeprom_write_block(&R.encoderRes, &E.encoderRes, sizeof (R.encoderRes));

		Status &=  ~SHORT_PUSH;			// Clear pushbutton status
		lcd_clrscr();
		lcd_gotoxy(1,1);				
//		lcd_puts_P("Value Stored");
						strcpy_P(buffer, general_text[store_change]);
						lcdPrintData(buffer,strlen(buffer));

		_delay_ms(1000);
		Menu_Mode |= CONFIG;			// We're NOT done, just backing off
		menu_level = 0;					// We are done with this menu level
		LCD_upd = FALSE;				// Make ready for next time
	}
}
*/

//--------------------------------------------------------------------
// Calibrate Submenu functions
//--------------------------------------------------------------------
void calibrate_menu_level2(void)
{
	static int16_t	current_selection;	// Keep track of current LCD menu selection
	static uint8_t LCD_upd = FALSE;		// Keep track of LCD update requirements
	char buffer[20];
	
	uint8_t cal_set;					// Determine whether CAL_SET0, CAL_SET1 or CAL_SET2

	if (menu_level == CAL_SET2_MENU || menu_level == CAL_SET5_MENU) 
		cal_set = 1;					// CAL_SET2_MENU
	else 
		cal_set = 0;					// CAL_SET0_MENU or CAL_SET1_MENU
		
	// These defines to aid readability of code
	#define CAL_BAD	0	// Input signal of insufficient quality for calibration
	#define CAL_FWD	1	// Good input signal detected, forward direction
	#define CAL_REV	2	// Good input signal detected, reverse direction (redundant)
	// Below variable can take one of the three above defined values, based on the
	// detected input "calibration" signal
	static uint8_t cal_sig_direction_quality;

	// Get Current value
	if (Shortwave)
		current_selection = R.cal_AD[cal_set].db10m;
	else
		current_selection = R.cal_AD_VHF[cal_set].db10m_VHF;

	// Selection modified by encoder. We remember last selection, even if exit and re-entry
	if (Status & ENC_CHANGE)
	{
		if (encOutput > 0)
		{
			current_selection++;
	    	// Reset data from Encoder
			Status &=  ~ENC_CHANGE;
			encOutput = 0;
		}
		else if (encOutput < 0)
		{
			current_selection--;
	  	  	// Reset data from Encoder
			Status &=  ~ENC_CHANGE;
			encOutput = 0;
		}
		// Indicate that an LCD update is needed
		LCD_upd = FALSE;
	}

	// Determine direction and level of calibration signal input
	// Check forward direction and sufficient level
	if (Shortwave)
	{
		if (((ad8307_adF - ad8307_adR) > CAL_INP_QUALITY) && (cal_sig_direction_quality != CAL_FWD))
		{
			cal_sig_direction_quality = CAL_FWD;
			LCD_upd = FALSE;		// Indicate that an LCD update is needed
		}
		// Check reverse direction and sufficient level
		else if (((ad8307_adR - ad8307_adF) > CAL_INP_QUALITY) && (cal_sig_direction_quality != CAL_REV))
		{
			cal_sig_direction_quality = CAL_REV;
			LCD_upd = FALSE;		// Indicate that an LCD update is needed
		}
		// Check insufficient level
		else if ((ABS((ad8307_adF - ad8307_adR)) <= CAL_INP_QUALITY) && (cal_sig_direction_quality != CAL_BAD))
		{
			cal_sig_direction_quality = CAL_BAD;
			LCD_upd = FALSE;		// Indicate that an LCD update is needed
		}
	}
	else
	{
		if (((ad8307_adF_VHF - ad8307_adR_VHF) > CAL_INP_QUALITY) && (cal_sig_direction_quality != CAL_FWD))
		{
			cal_sig_direction_quality = CAL_FWD;
			LCD_upd = FALSE;		// Indicate that an LCD update is needed
		}
		// Check reverse direction and sufficient level
		else if (((ad8307_adR_VHF - ad8307_adF_VHF) > CAL_INP_QUALITY) && (cal_sig_direction_quality != CAL_REV))
		{
			cal_sig_direction_quality = CAL_REV;
			LCD_upd = FALSE;		// Indicate that an LCD update is needed
		}
		// Check insufficient level
		else if ((ABS((ad8307_adF_VHF - ad8307_adR_VHF)) <= CAL_INP_QUALITY) && (cal_sig_direction_quality != CAL_BAD))
		{
			cal_sig_direction_quality = CAL_BAD;
			LCD_upd = FALSE;		// Indicate that an LCD update is needed
		}
	}
	
	// If LCD update is needed
	if (LCD_upd == FALSE)
	{
		LCD_upd = TRUE;							// We have serviced LCD

		// Keep Encoder Selection Within Bounds of the Menu Size
		int16_t max_value = 530;				// Highest permissible Calibration value in dBm * 10
		int16_t min_value = -100;				// Lowest permissible Calibration value in dBm * 10
		if(current_selection > max_value)
			current_selection = max_value;
		if(current_selection < min_value)
			current_selection = min_value;

		// Store Current value in running storage
		if (Shortwave)
			R.cal_AD[cal_set].db10m = current_selection;
		else
			R.cal_AD_VHF[cal_set].db10m_VHF = current_selection;

		lcdClear();
		lcdGotoXY(0,0);	

		if (menu_level == CAL_SET0_MENU)		// equals cal_set == 0
		{
			lcdPrintData("Single Level Cal:",17);
		}
		else if (menu_level == CAL_SET1_MENU)	// equals cal_set == 1
		{
			lcdPrintData("First Cal SetPoint:",19);
		}
		else if (menu_level == CAL_SET2_MENU)
		{
			lcdPrintData("Second Cal SetPoint:",20);
		}
		else if (menu_level == CAL_SET3_MENU)	// equals cal_set == 3, VHF single points
		{
			lcdPrintData("Single Level Cal:",17);
		}
		else if (menu_level == CAL_SET4_MENU)	// equals cal_set == 4, VHF
		{
			lcdPrintData("First Cal SetPoint:",19);
		}
		else if (menu_level == CAL_SET5_MENU)	// equals cal_set == 5, VHF
		{
			lcdPrintData("Second Cal SetPoint:",20);
		}

		lcdGotoXY(0,1);
		lcdPrintData("Adjust (dBm)->",14);
		// Format and print current value
		int16_t val_sub = current_selection;
		int16_t val = val_sub / 10;
		val_sub = val_sub % 10;
		if (current_selection < 0)
		{
			val*=-1;
			val_sub*=-1;
			sprintf(lcd_buf," -%1u.%01u",val, val_sub);
		}
		else
		{
			sprintf(lcd_buf," %2u.%01u",val, val_sub);
		}
		lcdPrintData(lcd_buf,strlen(lcd_buf));
		
		if (cal_sig_direction_quality == CAL_FWD)
		{
			lcdGotoXY(0,2);
			lcdPrintData(">Push to set<",13);
			lcdGotoXY(0,3);
			lcdPrintData("Signal detected",15);
		}
		else if (cal_sig_direction_quality == CAL_REV)
		{
			lcdGotoXY(0,2);
			lcdPrintData(">Push to set<",13);
			lcdGotoXY(0,3);
			lcdPrintData("Reverse detected",16);
		}
		else	// cal_sig_direction_quality == CAL_BAD
		{
			lcdGotoXY(0,2);
			lcdPrintData(">Push to exit<",14);
			lcdGotoXY(0,3);
			lcdPrintData("Poor signal quality",19);
		}
	}

	// Enact selection by saving in EEPROM
	if (Status & SHORT_PUSH)
	{
		Status &=  ~SHORT_PUSH;						// Clear pushbutton status
		lcdClear();
		lcdGotoXY(1,1);

		// Save modified value
		// If forward direction, then we calibrate for both, using the measured value for
		// in the forward direction only
		if (cal_sig_direction_quality == CAL_FWD)
		{
			if (menu_level == CAL_SET0_MENU)
			{
//				uint16_t thirty_dB = 983;			// Used for single shot calibration, 983 ADUs = 0,6 V at 2,5 Vref or 1200 ADUs = 0,75V at 2,5 Vref used for 25 mV/dB

				R.cal_AD[0].Fwd = ad8307_adF;
				R.cal_AD[0].Rev = ad8307_adF;
				eeprom_write_block(&R.cal_AD[0], &E.cal_AD[0], sizeof (R.cal_AD[0]));
				// Set second calibration point at 30 dB less
				R.cal_AD[1].db10m = R.cal_AD[0].db10m - 300;
				R.cal_AD[1].Fwd = R.cal_AD[0].Fwd - THIRTY_DB_VALUE;
				R.cal_AD[1].Rev = R.cal_AD[0].Fwd - THIRTY_DB_VALUE;
				eeprom_write_block(&R.cal_AD[1], &E.cal_AD[1], sizeof (R.cal_AD[1]));
			}
			else if (menu_level == CAL_SET3_MENU)
			{
//				uint16_t thirty_dB = 983;			// Used for single shot calibration, 983 ADUs = 0,6 V at 2,5 Vref or 1200 ADUs = 0,75V at 2,5 Vref used for 25 mV/dB

				R.cal_AD_VHF[0].Fwd_VHF = ad8307_adF_VHF;
				R.cal_AD_VHF[0].Rev_VHF = ad8307_adF_VHF;
				eeprom_write_block(&R.cal_AD_VHF[0], &E.cal_AD_VHF[0], sizeof (R.cal_AD_VHF[0]));
				// Set second calibration point at 30 dB less
				R.cal_AD_VHF[1].db10m_VHF = R.cal_AD_VHF[0].db10m_VHF - 300;
				R.cal_AD_VHF[1].Fwd_VHF = R.cal_AD_VHF[0].Fwd_VHF - THIRTY_DB_VALUE;
				R.cal_AD_VHF[1].Rev_VHF = R.cal_AD_VHF[0].Fwd_VHF - THIRTY_DB_VALUE;
				eeprom_write_block(&R.cal_AD_VHF[1], &E.cal_AD_VHF[1], sizeof (R.cal_AD_VHF[1]));				
			}
			else if (menu_level == CAL_SET1_MENU || menu_level == CAL_SET2_MENU)
			{
				
				R.cal_AD[cal_set].Fwd = ad8307_adF;
				R.cal_AD[cal_set].Rev = ad8307_adF;
				eeprom_write_block(&R.cal_AD[cal_set], &E.cal_AD[cal_set], sizeof (R.cal_AD[cal_set]));
			}
			else if (menu_level == CAL_SET4_MENU || menu_level == CAL_SET5_MENU)
			{
				R.cal_AD_VHF[cal_set].Fwd_VHF = ad8307_adF_VHF;
				R.cal_AD_VHF[cal_set].Rev_VHF = ad8307_adF_VHF;
				eeprom_write_block(&R.cal_AD_VHF[cal_set], &E.cal_AD_VHF[cal_set], sizeof (R.cal_AD_VHF[cal_set]));
			}
			strcpy_P(buffer, general_text[store_change]);
			lcdPrintData(buffer,strlen(buffer));
		}
		// If reverse, then we calibrate for reverse direction only
		else if (cal_sig_direction_quality == CAL_REV)
		{
			if (menu_level == CAL_SET0_MENU)
			{
				R.cal_AD[0].Rev = ad8307_adR;
				eeprom_write_block(&R.cal_AD[0].Rev, &E.cal_AD[0].Rev, sizeof (R.cal_AD[0].Rev));
				// Set second calibration point at 30 dB less
				R.cal_AD[1].Rev = R.cal_AD[0].Fwd - THIRTY_DB_VALUE;
				eeprom_write_block(&R.cal_AD[1].Rev, &E.cal_AD[1].Rev, sizeof (R.cal_AD[1].Rev));
			}
			else if (menu_level == CAL_SET3_MENU)
			{
				R.cal_AD_VHF[0].Rev_VHF = ad8307_adR_VHF;
				eeprom_write_block(&R.cal_AD_VHF[0].Rev_VHF, &E.cal_AD_VHF[0].Rev_VHF, sizeof (R.cal_AD_VHF[0].Rev_VHF));
				// Set second calibration point at 30 dB less
				R.cal_AD_VHF[1].Rev_VHF = R.cal_AD_VHF[0].Fwd_VHF - THIRTY_DB_VALUE;
				eeprom_write_block(&R.cal_AD_VHF[1].Rev_VHF, &E.cal_AD_VHF[1].Rev_VHF, sizeof (R.cal_AD_VHF[1].Rev_VHF));
			}
			else if (menu_level == CAL_SET1_MENU || menu_level == CAL_SET2_MENU)
			{
				R.cal_AD[cal_set].Rev = ad8307_adR;
				eeprom_write_block(&R.cal_AD[cal_set].Rev, &E.cal_AD[cal_set].Rev, sizeof (R.cal_AD[cal_set].Rev));
			}
			else if (menu_level == CAL_SET4_MENU || menu_level == CAL_SET5_MENU)
			{
				R.cal_AD_VHF[cal_set].Rev_VHF = ad8307_adR_VHF;
				eeprom_write_block(&R.cal_AD_VHF[cal_set].Rev_VHF, &E.cal_AD_VHF[cal_set].Rev_VHF, sizeof (R.cal_AD_VHF[cal_set].Rev_VHF));
			}
			strcpy_P(buffer, general_text[store_change]);
			lcdPrintData(buffer,strlen(buffer));
		}
		else	// cal_sig_direction_quality == CAL_BAD
		{
			strcpy_P(buffer, general_text[no_change]);
			lcdPrintData(buffer,strlen(buffer));
		}
		_delay_ms(1000);
		Menu_Mode |= CONFIG;				// We're NOT done, just backing off
		menu_level = CAL_MENU;				// We are done with this menu level
		LCD_upd = FALSE;					// Make ready for next time
	}
}



//--------------------------------------------------------------------
// Calibrate Menu functions
//--------------------------------------------------------------------
void calibrate_menu(void)
{
	static int8_t	current_selection;		// Keep track of current LCD menu selection
	static uint8_t LCD_upd = FALSE;			// Keep track of LCD update requirements

	// Selection modified by encoder.  We remember last selection, even if exit and re-entry
	if (Status & ENC_CHANGE)
	{
		if (encOutput > 0)
		{
			current_selection++;
	    	// Reset data from Encoder
			Status &=  ~ENC_CHANGE;
			encOutput = 0;
		}
		else if (encOutput < 0)
		{
			current_selection--;
	  	  	// Reset data from Encoder
			Status &=  ~ENC_CHANGE;
			encOutput = 0;
		}
		// Indicate that an LCD update is needed
		LCD_upd = FALSE;
	}

	// If LCD update is needed
	if (LCD_upd == FALSE)
	{
		LCD_upd = TRUE;						// We have serviced LCD

		// Keep Encoder Selection Within Bounds of the Menu Size
		uint8_t menu_size = calibrate_menu_size;
		while(current_selection >= menu_size)
			current_selection -= menu_size;
		while(current_selection < 0)
			current_selection += menu_size;

		lcdClear();
		
		// Print the Rotary Encoder scroll Menu
		//lcd_scroll_Menu((char**)calibrate_menu_items, menu_size, current_selection,1, 0,1);
		lcd_scroll_Menu(menu_calibrate_text, calibrate_menu_size, current_selection,1, 0, 1);
		lcdGotoXY(0,0);

		int16_t value=0;

		switch (current_selection)
		{
			case 0:
				lcdPrintData("Calibrate HF",12);
				lcdGotoXY(0,2);
				lcdPrintData("Calibrate using one",19);
				lcdGotoXY(6,3);
				lcdPrintData("accurate level",14);
				switch (current_selection)
				{
					case 0:
					case 1:
						value = R.cal_AD[0].db10m;
					break;
					case 2:
						value = R.cal_AD[1].db10m;
					break;
				}
				int16_t val_sub = value;
				int16_t val = val_sub / 10;
				val_sub = val_sub % 10;
				// Print value of currently indicated reference
				lcdGotoXY(16,0);
				if (value < 0)
				{
					val*=-1;
					val_sub*=-1;
					sprintf(lcd_buf,"-%1u.%01u",val, val_sub);
				}
				else
				{
					sprintf(lcd_buf,"%2u.%01u",val, val_sub);
				}
				lcdPrintData(lcd_buf,strlen(lcd_buf));
				break;
			case 1:
				lcdPrintData("Calibrate HF",12);
				lcdGotoXY(0,2);
				lcdPrintData("Set higher of two",17);
				lcdGotoXY(5,3);
				lcdPrintData("accurate levels",15);
				switch (current_selection)
				{
					case 0:
					case 1:
						value = R.cal_AD[0].db10m;
					break;
					case 2:
						value = R.cal_AD[1].db10m;
					break;
				}
				val_sub = value;
				val = val_sub / 10;
				val_sub = val_sub % 10;
				// Print value of currently indicated reference
				lcdGotoXY(16,0);
				if (value < 0)
				{
					val*=-1;
					val_sub*=-1;
					sprintf(lcd_buf,"-%1u.%01u",val, val_sub);
				}
				else
				{
					sprintf(lcd_buf,"%2u.%01u",val, val_sub);
				}
				lcdPrintData(lcd_buf,strlen(lcd_buf));
				break;
			case 2: 
				lcdPrintData("Calibrate HF",12);
				lcdGotoXY(0,2);
				lcdPrintData("Set lower of two",16);
				lcdGotoXY(5,3);
				lcdPrintData("accurate levels",15);
				lcdGotoXY(16,0);
				lcdPrintData(" --",3);
				break;

			case 3:
				lcdPrintData("Calibrate V/UHF",15);
				lcdGotoXY(0,2);
				lcdPrintData("Calibrate using one",19);
				lcdGotoXY(6,3);
				lcdPrintData("accurate level",14);
				switch (current_selection)
				{
					case 3:
					case 4:
						value = R.cal_AD_VHF[0].db10m_VHF;
					break;
					case 5:
						value = R.cal_AD_VHF[1].db10m_VHF;
					break;
				}
				val_sub = value;
				val = val_sub / 10;
				val_sub = val_sub % 10;
				// Print value of currently indicated reference
				lcdGotoXY(16,0);
				if (value < 0)
				{
					val*=-1;
					val_sub*=-1;
					sprintf(lcd_buf,"-%1u.%01u",val, val_sub);
				}
				else
				{
					sprintf(lcd_buf,"%2u.%01u",val, val_sub);
				}
				lcdPrintData(lcd_buf,strlen(lcd_buf));
				break;
			case 4:
				lcdPrintData("Calibrate V/UHF",15);
				lcdGotoXY(0,2);
				lcdPrintData("Set higher of two",17);
				lcdGotoXY(5,3);
				lcdPrintData("accurate levels",15);
				int16_t value=0;
				switch (current_selection)
				{
					case 3:
					case 4:
						value = R.cal_AD_VHF[0].db10m_VHF;
					break;
					case 5:
						value = R.cal_AD_VHF[1].db10m_VHF;
					break;
				}
				val_sub = value;
				val = val_sub / 10;
				val_sub = val_sub % 10;
				// Print value of currently indicated reference
				lcdGotoXY(16,0);
				if (value < 0)
				{
					val*=-1;
					val_sub*=-1;
					sprintf(lcd_buf,"-%1u.%01u",val, val_sub);
				}
				else
				{
					sprintf(lcd_buf,"%2u.%01u",val, val_sub);
				}
				lcdPrintData(lcd_buf,strlen(lcd_buf));
				break;
			case 5:
				lcdPrintData("Calibrate V/UHF",15);
				lcdGotoXY(0,2);
				lcdPrintData("Calibrate using one",19);
				lcdGotoXY(6,3);
				lcdPrintData("accurate level",14);
				lcdGotoXY(16,0);
				lcdPrintData(" --",3);
				break;
		}

		// Indicate Current value stored under the currently selected GainPreset
		// The "stored" value indication changes according to which GainPreset is currently selected.
		lcdGotoXY(0,0);
		lcdPrintData("Calibrate",9);
		if (current_selection < 3)				// current_selection = 0, 1 and 2
		{
			int16_t value=0;
			switch (current_selection)
			{
				case 0:
				case 1:
					value = R.cal_AD[0].db10m;
					break;
				case 2:
					value = R.cal_AD[1].db10m;
					break;
			}
			int16_t val_sub = value;
			int16_t val = val_sub / 10;
			val_sub = val_sub % 10;
			lcdGotoXY(16,0);					// Print value of currently indicated reference
			if (value < 0)
			{
				val*=-1;
				val_sub*=-1;
				sprintf(lcd_buf,"-%1u.%01u",val, val_sub);
			}
			else
			{
				sprintf(lcd_buf,"%2u.%01u",val, val_sub);
			}
			lcdPrintData(lcd_buf,strlen(lcd_buf));
		}
		else									// current_selection >= 3
		{
//			lcdGotoXY(16,0);					// stand hier nur bei HF Auswahl
//			lcdPrintData(" --",3);				// stand hier nur bei HF Auswahl
			int16_t value=0;
			switch (current_selection)
			{
				case 3:
				case 4:
				value = R.cal_AD_VHF[0].db10m_VHF;
				break;
				case 5:
				value = R.cal_AD_VHF[1].db10m_VHF;
				break;
			}
	
			int16_t val_sub = value;
			int16_t val = val_sub / 10;
			val_sub = val_sub % 10;
			lcdGotoXY(16,0);					// Print value of currently indicated reference
			if (value < 0)
			{
				val*=-1;
				val_sub*=-1;
				sprintf(lcd_buf,"-%1u.%01u",val, val_sub);
			}
			else
			{
				sprintf(lcd_buf,"%2u.%01u",val, val_sub);
			}
			lcdPrintData(lcd_buf,strlen(lcd_buf));
		}
	}

	// Enact selection
	if (Status & SHORT_PUSH)
	{
		Status &=  ~SHORT_PUSH;				// Clear pushbutton status
		switch (current_selection)
		{
			case 0:
				menu_level = CAL_SET0_MENU;
//				LCD_upd = FALSE;			// force LCD reprint
				break;
			case 1:
				menu_level = CAL_SET1_MENU;
//				LCD_upd = FALSE;			// force LCD reprint
				break;
			case 2:
				menu_level = CAL_SET2_MENU;
//				LCD_upd = FALSE;			// force LCD reprint
				break;
			case 3:
				menu_level = CAL_SET3_MENU;
//				LCD_upd = FALSE;			// force LCD reprint
				break;
			case 4:
				menu_level = CAL_SET4_MENU;
//				LCD_upd = FALSE;			// force LCD reprint
				break;
			case 5:
				menu_level = CAL_SET5_MENU;
//				LCD_upd = FALSE;			// force LCD reprint
				break;
			case 6:
				lcdClear();
				lcdGotoXY(1,1);				
				lcdPrintData("Done with Cal",13);
				_delay_ms(1000);
				Menu_Mode |= CONFIG;		// We're NOT done, just backing off
				menu_level = 0;				// We are done with this menu level
//				LCD_upd = FALSE;			// Make ready for next time
				break;
			default:
				lcdClear();
				lcdGotoXY(1,1);				
				lcdPrintData("Done with Cal",13);
				_delay_ms(1000);
//				Menu_Mode &=  ~CONFIG;		// We're done
				Menu_Mode |= CONFIG;		// We're NOT done, just backing off
				menu_level = 0;				// We are done with this menu level
//				LCD_upd = FALSE;			// Make ready for next time
				break;
		}
		LCD_upd = FALSE;			// force LCD reprint
	}
}


//--------------------------------------------------------------------
// Factory Reset with all default values
//--------------------------------------------------------------------
void factory_menu(void)
{
	static int8_t	current_selection;
	static uint8_t	LCD_upd = FALSE;		// Keep track of LCD update requirements
	char buffer[20];
	
	// Selection modified by encoder.  We remember last selection, even if exit and re-entry
	if (Status & ENC_CHANGE)
	{
		if (encOutput > 0)
		{
			current_selection++;
		}
		else if (encOutput < 0)
		{
			current_selection--;
		}
  	  	// Reset data from Encoder
		Status &=  ~ENC_CHANGE;
		encOutput = 0;

		// Indicate that an LCD update is needed
		LCD_upd = FALSE;
	}

	// If LCD update is needed
	if (LCD_upd == FALSE)
	{
		LCD_upd = TRUE;					// We have serviced LCD

		// Keep Encoder Selection Within Bounds of the Menu Size
		uint8_t menu_size = factory_menu_size;
		while(current_selection >= menu_size)
			current_selection -= menu_size;
		while(current_selection < 0)
			current_selection += menu_size;

		lcdClear();
		lcdGotoXY(0,0);
		lcdPrintData("All to Default?",15);

		// Print the Rotary Encoder scroll Menu
		//lcd_scroll_Menu((char**)factory_menu_items, menu_size, current_selection, 1, 0,1);
		lcd_scroll_Menu(menu_factory_text, factory_menu_size, current_selection,1, 0, 1);

		lcdGotoXY(0,3);
		lcdPrintData("Factory Reset Menu",18);
	}

	// Enact selection
	if (Status & SHORT_PUSH)
	{
		Status &=  ~SHORT_PUSH;			// Clear pushbutton status

		switch (current_selection)
		{
			case 0:
				lcdClear();
				lcdGotoXY(1,1);
				strcpy_P(buffer, general_text[no_change]);
				lcdPrintData(buffer,strlen(buffer));
				_delay_ms(1000);
				Menu_Mode |= CONFIG;// We're NOT done, just backing off
				menu_level = 0;		// We are done with this menu level
				LCD_upd = FALSE;	// Make ready for next time
				break;
			case 1: // Factory Reset
				// Force an EEPROM update:
				R.EEPROM_init_check = 0;		// Initialize eeprom to "factory defaults by indicating a mismatch
				eeprom_write_block(&R.EEPROM_init_check, &E.EEPROM_init_check, sizeof (R.EEPROM_init_check));
				lcdClear();
				lcdGotoXY(0,0);				
				lcdPrintData("Factory Reset",13);
				lcdGotoXY(0,1);
				lcdPrintData("All default",11);
				_delay_ms(1000);
				//while (1);					// Bye bye, Death by Watchdog
				//								// BUG, Watchdog is unreliable
				//								// problem related to teensy bootloader?
				asm volatile("jmp 0x00000");	// Soft Reset
			default:
				lcdClear();
				lcdGotoXY(1,1);				
				strcpy_P(buffer, general_text[no_change]);
				lcdPrintData(buffer,strlen(buffer));
				_delay_ms(1000);
				Menu_Mode &=  ~CONFIG;	// We're done
				menu_level = 0;			// We are done with this menu level
				LCD_upd = FALSE;		// Make ready for next time
				break;
		}
	}
}


//--------------------------------------------------------------------
// change the brightness of the LCD backlight 
//--------------------------------------------------------------------
void led_light_menu(void)
{
	static int8_t	current_selection;
	static uint8_t	LCD_upd = FALSE;		// Keep track of LCD update requirements
	char buffer[20];

	// Get Current value
	if		(R.LED_intensity == 0) current_selection = 0;
	else if (R.LED_intensity == 64) current_selection = 1;
	else if (R.LED_intensity == 127) current_selection = 2;
	else if (R.LED_intensity == 191) current_selection = 3;
	else if (R.LED_intensity == 255) current_selection = 4;
	else current_selection = 0;

	
	// Selection modified by encoder.  We remember last selection, even if exit and re-entry
	if (Status & ENC_CHANGE)
	{
		if (encOutput > 0)
		{
			current_selection++;
		}
		else if (encOutput < 0)
		{
			current_selection--;
		}
		// Reset data from Encoder
		Status &=  ~ENC_CHANGE;
		encOutput = 0;
		// Indicate that an LCD update is needed
		LCD_upd = FALSE;
	}
	// If LCD update is needed
	if (LCD_upd == FALSE)
	{
		LCD_upd = TRUE;					// We have serviced LCD
		// Keep Encoder Selection Within Bounds of the Menu Size
		uint8_t menu_size = led_menu_size;
		while(current_selection >= menu_size)
			current_selection -= menu_size;
		while(current_selection < 0)
			current_selection += menu_size;
		lcdClear();
		lcdGotoXY(0,0);
		lcdPrintData("LCD Backlight", 13);
		lcdGotoXY(0,1);
		lcdPrintData("Select",6);
		// Print the Rotary Encoder scroll Menu
		//lcd_scroll_Menu((char**)led_menu_items, menu_size, current_selection, 1, 0,1);
		lcd_scroll_Menu(led_menu_items_text, led_menu_size, current_selection, 1, 0, 1);
		switch (current_selection)
		{
			case 0:
				OCR0A = 0;
				R.LED_intensity = 0;
				//EXT_LCD_LED_PORT |= LCD_LED;			// Turn LCD backlight on
				break;
			case 1:
				OCR0A = 255;
				R.LED_intensity = 64;
				//EXT_LCD_LED_PORT &= ~LCD_LED;			// Turn LCD backligth off
				break;
			case 2:
				OCR0A= 127;
				R.LED_intensity = 127;
				break;
			case 3:
				OCR0A = 191;
				R.LED_intensity = 191;
				break;
			case 4:
				OCR0A = 255;
				R.LED_intensity = 255;
				break;
		}
//		lcdGotoXY(0,3);
//		lcdPrintData("LED brightness",14);
	}

	// Enact selection
	if (Status & SHORT_PUSH)
	{
		Status &=  ~SHORT_PUSH;								// Clear pushbutton status
		lcdClear();
		lcdGotoXY(1,1);
		if (eeprom_read_byte(&E.LED_intensity) != R.LED_intensity)//OCR0A)	// New Value
		{
			eeprom_write_block(&R.LED_intensity, &E.LED_intensity, sizeof (R.LED_intensity));
			strcpy_P(buffer, general_text[store_change]);
			lcdPrintData(buffer,strlen(buffer));
		}
		else												// nothing changed
		{
			strcpy_P(buffer, general_text[no_change]);
			lcdPrintData(buffer,strlen(buffer));
			R.LED_intensity = eeprom_read_byte(&E.LED_intensity);

		}
//		Menu_Mode &=  ~CONFIG;	// We're done
		Menu_Mode |= CONFIG;	// We're NOT done, just backing off
		menu_level = 0;					// We are done with this menu level
		LCD_upd = FALSE;				// Make ready for next time
		_delay_ms(1000);
	}
}
//
//--------------------------------------------------------------------
// Manage the first level of Menus
//--------------------------------------------------------------------
//
void menu_level0(void)
{
	static int8_t	current_selection;	// Keep track of current LCD menu selection
	static uint8_t	LCD_upd = FALSE;	// Keep track of LCD update requirements

	// Selection modified by encoder.  We remember last selection, even if exit and re-entry
	if (Status & ENC_CHANGE)
	{
		if (encOutput > 0)
		{
			current_selection++;
	    	// Reset data from Encoder
			Status &=  ~ENC_CHANGE;
			encOutput = 0;
		}
		else if (encOutput < 0)
		{
			current_selection--;
	  	  	// Reset data from Encoder
			Status &=  ~ENC_CHANGE;
			encOutput = 0;
		}
		// Indicate that an LCD update is needed
		LCD_upd = FALSE;
	}

	if (LCD_upd == FALSE)				// Need to update LCD
	{
		LCD_upd = TRUE;					// We have serviced LCD

		// Keep Encoder Selection Within Bounds of the Menu Size
		uint8_t menu_size = level0_menu_size;
		while(current_selection >= menu_size)
			current_selection -= menu_size;
		while(current_selection < 0)
			current_selection += menu_size;

		lcdClear();
		lcdGotoXY(0,0);
		lcdPrintData("Config Menu:",12);

		// Print the Menu
		//lcd_scroll_Menu((char**)level0_menu_items, menu_size, current_selection,1, 0,3);
		lcd_scroll_Menu(menu_level0_text, level0_menu_size, current_selection, 1, 0, 3);
		//lcdPrintData(buffer,strlen(buffer));
	}

	if (Status & SHORT_PUSH)
	{
		Status &= ~SHORT_PUSH;			// Clear pushbutton status

		switch (current_selection)
		{
			case 0: // SWR Alarm Threshold Set
				menu_level = SWR_ALARM_THRESHOLD;
				LCD_upd = FALSE;		// force LCD reprint
				break;

			case 1: // SWR Alarm Power Threshold Set
				menu_level = SWR_ALARM_PWR_THRESHOLD;
				LCD_upd = FALSE;		// force LCD reprint
				break;

			case 2: // PEP sampling period select
				menu_level = PEP_MENU;
				LCD_upd = FALSE;		// force LCD reprint
				break;

			case 3: // Screensaver Threshold
				menu_level = SCREENTHRESH_MENU;
				LCD_upd = FALSE;		// force LCD reprint
				break;

			case 4: // Scale Range Set
				menu_level = SCALERANGE_MENU;
				LCD_upd = FALSE;		// force LCD reprint
				break;

			case 5: // Serial Data
				menu_level = SERIAL_MENU;
				LCD_upd = FALSE;		// force LCD reprint
				break;

			case 6: // Calibrate
				menu_level = CAL_MENU;
				LCD_upd = FALSE;		// force LCD reprint
				break;

			case 7: // Display Debug stuff
				menu_level = DEBUG_MENU;
				LCD_upd = FALSE;		// force LCD reprint
				break;

			case 8: // Factory Reset
				menu_level = FACTORY_MENU;
				LCD_upd = FALSE;		// force LCD reprint
				break;
			case 9:	// LED brightness
				menu_level = LED_LIGHT;
				LCD_upd = FALSE;		// force LCD reprint
				break;
			//case x:// Encoder Resolution
			//	menu_level = ENCODER_MENU;
			//	LCD_upd = FALSE;		// force LCD reprint
			//	break;

			default: // Exit
				lcdClear();
				lcdGotoXY(1,1);
		   		lcdPrintData("Return from Menu",16);
				current_selection = 0;
				Menu_Mode &=  ~CONFIG;	// We're done
				LCD_upd = FALSE;		// Make ready for next time
		}
	}
}


//
//--------------------------------------------------------------------
// Scan the Configuraton Menu Status and delegate tasks accordingly
//--------------------------------------------------------------------
//
void PushButtonMenu(void)
{
	// Select which menu level to manage
	if (menu_level == 0) menu_level0();
	else if (menu_level == SWR_ALARM_THRESHOLD) swr_alarm_threshold_menu();
	else if (menu_level == SWR_ALARM_PWR_THRESHOLD) swr_alarm_power_threshold_menu();
	else if (menu_level == PEP_MENU) pep_menu();
	else if (menu_level == SCREENTHRESH_MENU) screenthresh_menu();
	else if (menu_level == SCALERANGE_MENU) scalerange_menu();
	else if (menu_level == SCALE_SET0_MENU) scalerange_menu_level2();
	else if (menu_level == SCALE_SET1_MENU) scalerange_menu_level2();
	else if (menu_level == SCALE_SET2_MENU) scalerange_menu_level2();
	else if (menu_level == SERIAL_MENU) serial_menu();
	//else if (menu_level == ENCODER_MENU) encoder_menu();
	else if (menu_level == CAL_MENU) calibrate_menu();
	else if (menu_level == CAL_SET0_MENU) calibrate_menu_level2();
	else if (menu_level == CAL_SET1_MENU) calibrate_menu_level2();
	else if (menu_level == CAL_SET2_MENU) calibrate_menu_level2();
	else if (menu_level == CAL_SET3_MENU) calibrate_menu_level2();
	else if (menu_level == CAL_SET4_MENU) calibrate_menu_level2();
	else if (menu_level == CAL_SET5_MENU) calibrate_menu_level2();
	else if (menu_level == DEBUG_MENU) debug_menu();
	else if (menu_level == FACTORY_MENU) factory_menu();
	else if (menu_level == LED_LIGHT) led_light_menu();
}
